// ExcelDriver.hpp
//
// Excel driver class. This class is for VISUALISATION only
//
// (C) Datasim Education BV 2003 - 2012
//

#ifndef ExcelDriver_HPP
#define ExcelDriver_HPP

// !!!! RIGHT VERSION OF OFFICE!!!!!!
#include "ExcelImports.cpp" // Contains all versions of Office
// STL list class.
#include <list>
using namespace std;
#include <boost/numeric/ublas/matrix.hpp>		// The matrix class.
#include <boost/numeric/ublas/vector.hpp>
namespace u = boost::numeric::ublas;

// Excel driver class definition. Contains functionality to add charts 
// and matrices. Hides all COM details. COM exceptions are re-thrown
// as STL strings.

class ExcelDriver
{
private:
	// Private member data.
	Excel::_ApplicationPtr xl;		// Pointer to Excel.
	long curDataColumn;				// Current column of 'Chart Data'

	// Writes label and vector to cells in horizontal direction.
	void ToSheetHorizontal( Excel::_WorksheetPtr sheet,
					long sheetRow, long sheetColumn,
					const std::string& label,const u::vector<double> & values );

	// Writes label and vector to cells in vertical direction.
	void ToSheetVertical( Excel::_WorksheetPtr sheet,
					long sheetRow, long sheetColumn,
					const std::string& label, const u::vector<double> & values );


	// Throw COM error as string.
	void ThrowAsString( _com_error & error );

public:
	// Constructor. Starts Excel in invisible mode.
	ExcelDriver();

	// Destructor.
	virtual ~ExcelDriver();
	

	// Access to single, shared instance of ExcelDriver (singleton).
	static ExcelDriver& Instance();


	// Add Matrix to the spreadsheet with row and column labels.
	void AddMatrix(const std::string& sheetName, const u::matrix<double>& matrix, 
			   const list<std::string>& rowLabels, const list<std::string>& columnLabels);

	void MakeVisible( bool b);

	// For debugging, for example
	void printStringInExcel(const string& s, long rowNumber, long colNumber, 
						const string& sheetName);
	void printStringInExcel(const list<string>& s, long rowNumber, long colNumber, 
						const string& sheetName);


};
u::vector <double> createVector(const u::matrix<double>& mat, long row);
void printMatrixInExcel(const u::matrix<double>& myMatrix,
				 const u::vector<double>& xarr, const u::vector<double>& yarr, 
				 const std::string& SheetName, bool show);
#endif