// GenericParabolicPde.hpp
//
// The 2-factor BS Pde to create the Pde and the Domain.
//
// (C) Datasim Education BC 2009-2012
//

#ifndef GenericParabolicPde_hpp
#define GenericParabolicPde_hpp

#include "TwoFactorPde.hpp"
#include "TwoFactorPdeDomain.hpp"
#include "GeneralInclude.hpp"
//#include <windows.h>

namespace GenericParabolicPde
{

	// Domain information
	double T;
	double xMax, yMax;
	double rho;

	double diffusionX(double x, double y, double t)
	{		

		return 1.0;
	}

		
	double diffusionY(double x, double y, double t)
	{

		return 1.0;
	}

	double convectionX(double x, double y, double t)
	{
		return 0.0;

	}

	double convectionY(double x, double y, double t)
	{
		return 0.0;
	}

	double termXY(double x, double y, double t)
	{
		return rho;
	}

	double termFree(double x, double y, double t)
	{

		return 0.0;
	}

	double termInhomogeneous(double x, double y, double t)
	{
		return -5.0;
	}

	// Factories

	void createTwoFactorPde(TwoFactorPde<double>& pde)
	{
		
		// Initialise all functions

		/*
		// U_t = a11U_Domain + a22U_yy + b1U_x + b2U_y + cU_xy + dU + F; // f = f(x,y,t)
		boost::function<T (T,T,T)> a11;
		boost::function<T (T,T,T)> a22;
		boost::function<T (T,T,T)> c;
		boost::function<T (T,T,T)> b1;
		boost::function<T (T,T,T)> b2;
		boost::function<T (T,T,T)> d;
		boost::function<T (T,T,T)> F;*/

		pde.a11 = &GenericParabolicPde::diffusionX;
		pde.a22 = &GenericParabolicPde::diffusionY;
		pde.c = &GenericParabolicPde::termXY;
		pde.b1 = &GenericParabolicPde::convectionX;
		pde.b2 = &GenericParabolicPde::convectionY;
		pde.d = &GenericParabolicPde::termFree;
		pde.F = &GenericParabolicPde::termInhomogeneous;
//cout << "F " << result.F(1, 1, 0); int g; cin >> g;
	}

	double IC(double x, double y)
	{
		return x*x + y*y + rho*x*y;
	}

	double BCLower(double x, double t)
	{

		return x*x-t;
	}


	double BCUpper(double x, double t)
	{

		return x*x + yMax*yMax + rho*x*yMax - t;

	}

	double BCLeft(double y, double t)
	{ // y is S2
		
		return y*y - t;
	}
	
	double BCRight(double y, double t)
	{
		return xMax*xMax + y*y + rho*xMax*y - t;
	}

	// Factory
	void createTwoFactorPdeDomain(TwoFactorPdeDomain<double>& pdeDomain)
	{
		
		pdeDomain.rx = Range<double>(0.0, xMax);
		pdeDomain.ry = Range<double>(0.0, yMax);
		pdeDomain.rt = Range<double>(0.0, T);

		pdeDomain.LeftBC = &GenericParabolicPde::BCLeft;
		pdeDomain.RightBC = &GenericParabolicPde::BCRight;
		pdeDomain.UpperBC = &GenericParabolicPde::BCUpper;
		pdeDomain.LowerBC = &GenericParabolicPde::BCLower;

		pdeDomain.IC = &IC;

	}
} // End of namespace

#endif