// TwoFactorPdeDomain.hpp
//
// The class containing description of (x,y,t) geometry and related
// auxiliary functions:
// 
//		1. 2d domain definition (3 ranges), it's a rectangle
//		2. Boundary conditions in space (lower and upper in x and y)
//		3. Initial condition
//
// (C) Datasim Education BV 2010
//

#ifndef TwoFactorPdeDomain_hpp
#define TwoFactorPdeDomain_hpp

#include "Range.cpp"
#include <boost/function.hpp>

template <typename T> struct TwoFactorPdeDomain
{
	//	1. Domain
	Range<T> rx;	
	Range<T> ry;
	Range<T> rt;

	//	2. Boundary conditions, anticlockwise
	boost::function<T (T x,T t)> LowerBC;
	boost::function<T (T x,T t)> UpperBC;
	boost::function<T (T y,T t)> LeftBC;
	boost::function<T (T y,T t)> RightBC;

	//	3. Initial condition
	boost::function<T (T x,T y)> IC;
};

#endif